/// <reference types="koa__router" />
import { Middleware } from 'koa';
import { IOptions } from 'glob';
import { IAwilixControllerBuilder } from 'awilix-router-core';
import Router from '@koa/router';
/**
 * Constructor type.
 */
export type ConstructorOrControllerBuilder = (new (...args: Array<any>) => any) | IAwilixControllerBuilder;
/**
 * Registers one or multiple decorated controller classes.
 *
 * @param ControllerClass One or multiple "controller" classes
 *        with decorators to register
 */
export declare function controller(ControllerClass: ConstructorOrControllerBuilder | Array<ConstructorOrControllerBuilder>): Middleware;
/**
 * Imports and prepares controllers for the given pattern, applying them to the supplied router
 *
 * @param router
 * @param pattern
 * @param opts
 */
export declare function importControllers(router: Router, pattern: string, opts?: IOptions): void;
/**
 * Loads controllers for the given pattern and returns a koa-compose'd Middleware
 * This return value must be used with `Koa.use`, and is incompatible with `Router.use`
 *
 * @param pattern
 * @param opts
 * @param router
 */
export declare function loadControllers(pattern: string, opts?: IOptions, router?: Router): Middleware;
